Type: Package
Package: densratio
Title: Density Ratio Estimation
Version: 0.3.1
Authors@R: 
    person("Koji", "Makiyama", , "hoxo.smile@gmail.com", role = c("aut", "cre"))
Description: Density ratio estimation.  The estimated density ratio
    function can be used in many applications such as anomaly detection,
    change-point detection, covariate shift adaptation.  The implemented
    methods are uLSIF (Hido et al. (2011)
    <doi:10.1007/s10115-010-0283-2>), RuLSIF (Yamada et al. (2011)
    <doi:10.1162/NECO_a_00442>), and KLIEP (Sugiyama et al. (2007)
    <doi:10.1007/s10463-008-0197-x>).
License: MIT + file LICENSE
URL: https://github.com/hoxo-m/densratio
BugReports: https://github.com/hoxo-m/densratio/issues
Imports: utils
Suggests: graphics, knitr, mvtnorm, rmarkdown, stats, testthat
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-29 12:43:32 UTC; akagi
Author: Koji Makiyama [aut, cre]
Maintainer: Koji Makiyama <hoxo.smile@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-29 13:10:11 UTC
Built: R 4.6.0; ; 2026-05-01 20:12:05 UTC; unix
