# Generated by roxygen2: do not edit by hand

S3method(print,KLIEP)
S3method(print,RuLSIF)
S3method(print,densratio)
S3method(print,uLSIF)
export(KLIEP)
export(RuLSIF)
export(densratio)
export(uLSIF)
importFrom(utils,str)
